#ifndef SIGNATURE_H_INCLUDED_
#define SIGNATURE_H_INCLUDED_

// This file is part of Enblend.
// Licence details can be found in the file COPYING.

#include <numeric>

extern const std::string command;

class Signature
{
public:
    Signature(): checksum_(000000014671U), neg_checksum_(0U) {}

    const wchar_t* message() const
    {
        return L"\103\157\155\160\151\154\145\144"
            L"\040\157\156\040\146\145\144\157"
            L"\162\141\062\056\163\063\071\060"
            L"\056\142\157\163\056\162\145\144"
            L"\150\141\164\056\143\157\155\040"
            L"\142\171\040\155\157\143\153\142"
            L"\165\151\154\144\040\157\156\040"
            L"\106\162\151\054\040\112\141\156"
            L"\040\060\066\040\062\060\061\065"
            L"\054\040\061\066\072\065\064\072"
            L"\064\062\056";
    }

    void initialize()
    {
#ifdef DEBUG_FORCE_SIGNATURE_CHECK_FAILURE
        checksum_++;
        neg_checksum_ = checksum_;
#else
        neg_checksum_ = ~checksum_;
#endif
    }

    void check() const
    {
#if DEBUG_SIGNATURE_CHECK
        if (checksum_ != ~neg_checksum_)
        {
            std::cerr <<
                "+ static checksum " << checksum_ <<
                " does not match static shadow checksum " << ~neg_checksum_ << "\n";
        }
        if (generate_checksum() != checksum_)
        {
            std::cerr <<
                "+ dynamic checksum " << generate_checksum() <<
                " does not match static checksum " << checksum_ << ", where\n" <<
                "+     message is <" << message() << ">\n";
        }
#endif

        if (generate_checksum() != checksum_ || checksum_ != ~neg_checksum_)
        {
            std::cerr << command.c_str(); // MSVC chokes without c_str()
#ifdef WANT_AGGRESSIVE_SIGNATURE_CHECK
            std::cerr << "\072\040\164\141\155\160\145\162"
            "\145\144\040\142\151\156\141\162"
            "\171\012";
            exit(1);
#else
            std::cerr << "\072\040\167\141\162\156\151\156"
            "\147\072\040\163\151\147\156\141"
            "\164\165\162\145\040\143\150\145"
            "\143\153\040\146\141\151\154\145"
            "\144\012";
#endif
        }
    }

    unsigned generate_checksum() const
    {
        const wchar_t* m = message();
        return m == NULL ? 0U : std::accumulate(m, m + wcslen(m), 0U) & 037777777777U;
    }

private:
    unsigned checksum_;
    unsigned neg_checksum_;
};

#endif // SIGNATURE_H_INCLUDED_
